LIBRARY      PABLIO
DESCRIPTION  'PABLIO   Portable Audio Blocking I/O'

EXPORTS
    ; Explicit exports can go here
	Pa_Initialize                   @1
	Pa_Terminate                    @2
	Pa_GetHostError                 @3
	Pa_GetErrorText                 @4
	Pa_CountDevices                 @5
	Pa_GetDefaultInputDeviceID      @6
	Pa_GetDefaultOutputDeviceID     @7
	Pa_GetDeviceInfo                @8
	Pa_OpenStream                   @9
	Pa_OpenDefaultStream            @10
	Pa_CloseStream                  @11
	Pa_StartStream                  @12
	Pa_StopStream                   @13
	Pa_StreamActive                 @14
	Pa_StreamTime                   @15
	Pa_GetCPULoad                   @16
	Pa_GetMinNumBuffers             @17
	Pa_Sleep                        @18

	OpenAudioStream                 @19
	CloseAudioStream                @20
	WriteAudioStream                @21
	ReadAudioStream                 @22

	Pa_GetSampleSize                @23

   ;123456789012345678901234567890123456
   ;000000000111111111122222222223333333


